onload = function() {
  var start = document.getElementById("start");
  var stop = document.getElementById("stop");
  var hosts = document.getElementById("hosts");
  var port = document.getElementById("port");
  var portLine = document.getElementById("portLine");
  var directory = document.getElementById("directory");

  var tcpServer = chrome.sockets.tcpServer;
  var tcpSocket = chrome.sockets.tcp;

  var serverSocketId = null;
  var filesMap = {};
  var send2CRA;

  var stringToUint8Array = function(string) {
    var buffer = new ArrayBuffer(string.length);
    var view = new Uint8Array(buffer);
    for (var i = 0; i < string.length; i++) {
      view[i] = string.charCodeAt(i);
    }
    return view;
  };

  var arrayBufferToString = function(buffer) {
    var str = '';
    var uArrayVal = new Uint8Array(buffer);
    for (var s = 0; s < uArrayVal.length; s++) {
      str += String.fromCharCode(uArrayVal[s]);
    }
    return str;
  };

  var logToScreen = function(log) {
    logger.textContent += log + "\n";
    logger.scrollTop = logger.scrollHeight;
  };

  var destroySocketById = function(socketId) {
    tcpSocket.disconnect(socketId, function() {
      tcpSocket.close(socketId);
    });
  };

  var closeServerSocket = function() {
    if (serverSocketId) {
      tcpServer.close(serverSocketId, function() {
        if (chrome.runtime.lastError) {
          console.warn("chrome.sockets.tcpServer.close:", chrome.runtime.lastError);
        }
      });
    }

    tcpServer.onAccept.removeListener(onAccept);
    tcpSocket.onReceive.removeListener(onReceive);
  };

  var sendReplyToSocket = function(socketId, buffer, keepAlive) {
    // verify that socket is still connected before trying to send data
    tcpSocket.getInfo(socketId, function(socketInfo) {
      if (!socketInfo.connected) {
        destroySocketById(socketId);
        return;
      }

      tcpSocket.setKeepAlive(socketId, keepAlive, 1, function() {
        if (!chrome.runtime.lastError) {
          tcpSocket.send(socketId, buffer, function(writeInfo) {
            console.log("WRITE", writeInfo);

            if (!keepAlive || chrome.runtime.lastError) {
              destroySocketById(socketId);
            }
          });
        }
        else {
          console.warn("chrome.sockets.tcp.setKeepAlive:", chrome.runtime.lastError);
          destroySocketById(socketId);
        }
      });
    });
  };

  var getResponseHeader = function(file, errorCode, keepAlive) {
    var httpStatus = "HTTP/1.0 200 OK";
    var contentType = "text/html";
    var contentLength = 0;

    if (!file || errorCode) {
      httpStatus = "HTTP/1.0 " + (errorCode || 404) + " Not Found";
    }
    else {
      contentType = file.type || contentType;
      contentLength = file.size;
    }

    var lines = [
      httpStatus,
      "Content-length: " + contentLength,
      "Content-type:" + contentType
    ];

    if (keepAlive) {
      lines.push("Connection: keep-alive");
    }

    return stringToUint8Array(lines.join("\n") + "\n\n");
  };

  var getResponseHeader = function(file, errorCode, keepAlive) {
    var httpStatus = "HTTP/1.0 200 OK";
    var contentType = "text/plain";
    var contentLength = 0;

    if (!file || errorCode) {
      httpStatus = "HTTP/1.0 " + (errorCode || 404) + " Not Found";
    }
    else {
      contentType = file.type || contentType;
      contentLength = file.size;
    }

    var lines = [
      httpStatus,
      "Content-length: " + contentLength,
      "Content-type:" + contentType
    ];

    if (keepAlive) {
      lines.push("Connection: keep-alive");
    }

    return stringToUint8Array(lines.join("\n") + "\n\n");
  };

  var getResponseHeaderBuffer = function(fileBuffer, errorCode, keepAlive) {
    var httpStatus = "HTTP/1.0 200 OK";
    var contentType = "text/html";
    var contentLength = 0;

    contentLength = fileBuffer.byteLength;

    var lines = [
      httpStatus,
      "Content-length: " + contentLength,
      "Content-type:" + contentType
    ];

    if (keepAlive) {
      lines.push("Connection: keep-alive");
    }

    return stringToUint8Array(lines.join("\n") + "\n\n");
  };

  var getErrorHeader = function(errorCode, keepAlive) {
    return getResponseHeader(null, errorCode, keepAlive);
  };

  var getSuccessHeader = function(file, keepAlive) {
    return getResponseHeader(file, null, keepAlive);
  };

  var getSuccessHeaderBuffer = function(fileBuffer, keepAlive) {
    return getResponseHeaderBuffer(fileBuffer, null, keepAlive);
  };

  var writeErrorResponse = function(socketId, errorCode, keepAlive) {
    console.info("writeErrorResponse:: begin... ");

    var header = getErrorHeader(errorCode, keepAlive);
    console.info("writeErrorResponse:: Done setting header...");
    var outputBuffer = new ArrayBuffer(header.byteLength);
    var view = new Uint8Array(outputBuffer);
    view.set(header, 0);
    console.info("writeErrorResponse:: Done setting view...");

    sendReplyToSocket(socketId, outputBuffer, keepAlive);

    console.info("writeErrorResponse::filereader:: end onload...");
    console.info("writeErrorResponse:: end...");
  };

  var write200Response = function(socketId, file, keepAlive) {
    var header = getSuccessHeader(file, keepAlive);
    var outputBuffer = new ArrayBuffer(header.byteLength + file.size);
    var view = new Uint8Array(outputBuffer);
    view.set(header, 0);

    var fileReader = new FileReader();
    fileReader.onload = function(e) {
      view.set(new Uint8Array(e.target.result), header.byteLength);
      sendReplyToSocket(socketId, outputBuffer, keepAlive);
      document.getElementById('lblMsg').innerHTML="Send reply to socket " + socketId;
    };

    fileReader.readAsArrayBuffer(file);
  };

  var write200ResponseBuffer = function(socketId, fileBuffer, keepAlive) {
    var header = getSuccessHeaderBuffer(fileBuffer, keepAlive);
    var outputBuffer = new ArrayBuffer(header.byteLength + fileBuffer.byteLength);
    var view = new Uint8Array(outputBuffer);
    view.set(header, 0);
    view.set(fileBuffer, header.byteLength);

    sendReplyToSocket(socketId, outputBuffer, keepAlive);
    if (send2CRA) {
        document.getElementById('lblMsg').innerHTML="Getting tax data from CRA...";
        setTimeout(function(){closeServerSocket();serverSocketId = null;}, 1000);
        TDDResend = true;
        connect2NTP(2);
    }
  };

  var onAccept = function(acceptInfo) {
    tcpSocket.setPaused(acceptInfo.clientSocketId, false);

    if (acceptInfo.socketId != serverSocketId)
      return;

    console.log("ACCEPT", acceptInfo);
  };

  var onReceive = function(receiveInfo) {
    console.log("READ", receiveInfo);
    var socketId = receiveInfo.socketId;

    // Parse the request.
    var data = arrayBufferToString(receiveInfo.data);
    // we can only deal with GET requests
    if (data.indexOf("GET ") !== 0) {
      // close socket and exit handler
      destroySocketById(socketId);
      return;
    }

    var keepAlive = false;
    if (data.indexOf("Connection: keep-alive") != -1) {
      keepAlive = true;
    }
    var uriEnd = data.indexOf(" ", 4);
    if (uriEnd < 0) {
	// throw a wobbler
	 return; }
    var uri = data.substring(4, uriEnd);
    send2CRA = false;
    if (uri.substring(0,4) == "/?t=") {
        tokenFromCRA = decodeURIComponent(uri.substring(4, uri.length));
        send2CRA = true;
    }
/*
    // strip query string
    var q = uri.indexOf("?");
    if (q != -1) {
      uri = uri.substring(0, q);
    }

    var file = filesMap[uri];
    if (!!file == false) {
      console.warn("File does not exist..." + uri);
      writeErrorResponse(socketId, 404, keepAlive);
      return;
    }
    logToScreen("GET 200 " + uri);
*/
    var element = document.getElementById('htmlResp');
    var HTMLDoc = element.contentDocument;
    var fileString = HTMLDoc.documentElement.outerHTML;
    console.log(fileString);
    var fileBuffer = stringToUint8Array(fileString);

//    write200Response(socketId, file, keepAlive);
    write200ResponseBuffer(socketId, fileBuffer, keepAlive);

  };

  directory.onchange = function(e) {
    closeServerSocket();

    var files = e.target.files;

    for (var i = 0; i < files.length; i++) {
      //remove the first first directory
      var path = files[i].webkitRelativePath;
      if (path && path.indexOf("/") >= 0) {
        filesMap[path.substr(path.indexOf("/"))] = files[i];
      } else {
        filesMap["/" + files[i].fileName] = files[i];
      }
    }

    start.disabled = false;
    stop.disabled = true;
    directory.disabled = true;
  };

  start.onclick = function() {
    portLine.style.visibility = "hidden";

    tcpServer.create({}, function(socketInfo) {
      serverSocketId = socketInfo.socketId;

      document.getElementById('lblMsg').innerHTML="Listening to port " + port.value;
      tcpServer.listen(serverSocketId, hosts.value, parseInt(port.value, 10), 50, function(result) {
        console.log("LISTENING:", result);

        tcpServer.onAccept.addListener(onAccept);
        tcpSocket.onReceive.addListener(onReceive);
      });
    });

    directory.disabled = true;
    stop.disabled = false;
    start.disabled = true;
    showCRAWeb();
  };

  stop.onclick = function() {
    portLine.style.visibility = "initial";
    directory.disabled = false;
    stop.disabled = true;
    start.disabled = false;
    closeServerSocket();
    document.getElementById('lblMsg').innerHTML="No connection";
    TDDResend = false;
 };
/*
  chrome.system.network.getNetworkInterfaces(function(interfaces) {
    for (var i in interfaces) {
      var interface = interfaces[i];
      var opt = document.createElement("option");
      opt.value = interface.address;
      opt.innerText = interface.name + " - " + interface.address;
      hosts.appendChild(opt);
    }
  });
*/
};
